package cn.fw.valhalla.common.utils;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.Calendar;
import java.util.Date;
import java.util.Objects;

/**
 * 日期处理工具
 * <p>
 *
 * @author kurisu
 */
public final class DateUtil {

    static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * 默认构造器
     */
    private DateUtil() {
        throw new UnsupportedOperationException();
    }


    public static Date parse(String date) {
        try {
            return sdf.parse(date);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 开始日期处理
     */
    public static Date startDate(final Date date) {
        if (date == null) {
            return null;
        }
        return localDateTime2Date(date2LocalDate(date).atStartOfDay());
    }

    /**
     * 最结束日期处理
     */
    public static Date endDate(final Date date) {
        if (date == null) {
            return null;
        }
        return localDateTime2Date(date2LocalDate(date).plusDays(1).atStartOfDay());
    }

    /**
     * 处理日期，保留年月日
     */
    public static LocalDate date2LocalDate(final Date date) {
        if (date == null) {
            return null;
        }
        return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault()).toLocalDate();
    }

    public static LocalTime date2LocalTime(final Date date) {
        if (date == null) {
            return null;
        }
        return LocalTime.of(date.getHours(), date.getMinutes(), date.getSeconds());
    }

    public static LocalDateTime date2LocalDateTime(final Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        return instant.atZone(zoneId).toLocalDateTime();
    }

    /**
     * convert LocalDateTime to Date
     */
    public static Date localDateTime2Date(final LocalDateTime dateTime) {
        return Date.from(dateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * convert LocalDate to Date
     */
    public static Date localDate2Date(final LocalDate localDate) {
        if (localDate == null) {
            return null;
        }
        return Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
    }

    public static Date getBeginInTime(Date inTime) {
        if (inTime == null) {
            return null;
        }
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTime();
    }

    public static Date getEndInTime(Date inTime) {
        if (inTime == null) {
            return null;
        }
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        return c.getTime();
    }

    /**
     * 得到本周周一
     *
     * @return
     */
    public static Date getMondayOfThisWeek(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        int day_of_week = c.get(Calendar.DAY_OF_WEEK) - 1;
        if (day_of_week == 0) {
            day_of_week = 7;
        }
        c.add(Calendar.DATE, -day_of_week + 1);
        return c.getTime();
    }

    /**
     * 得到本周周日
     *
     * @return
     */
    public static Date getSundayOfThisWeek(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        int day_of_week = c.get(Calendar.DAY_OF_WEEK) - 1;
        if (day_of_week == 0) {
            day_of_week = 7;
        }
        c.add(Calendar.DATE, -day_of_week + 7);
        return c.getTime();
    }

    /**
     * 得到月初
     *
     * @return
     */
    public static Date getMonthFirstDay(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        c.add(Calendar.MONTH, 0);
        c.set(Calendar.DAY_OF_MONTH, 1);
        return c.getTime();
    }

    /**
     * 获取当月月初时间
     */
    public static Date getMonthFirstDay() {
        return DateUtil.localDate2Date(LocalDateTime.now()
                .with(TemporalAdjusters.firstDayOfMonth()).toLocalDate());
    }

    /**
     * 得到月末
     *
     * @return
     */
    public static Date getMonthEndDay(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        c.add(Calendar.MONTH, 1);
        c.set(Calendar.DAY_OF_MONTH, 0);
        return c.getTime();
    }

    /**
     * 得到月末
     *
     * @return
     */
    public static Date getMonthEndDay() {
        return DateUtil.localDate2Date(LocalDateTime.now()
                .with(TemporalAdjusters.lastDayOfMonth()).toLocalDate());
    }

    /**
     * 得到年初
     *
     * @return
     */
    public static Date getYearFirstDay(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        int x = c.get(Calendar.YEAR);
        try {
            return sdf.parse(x + "-01" + "-01");
        } catch (Exception e) {

        }
        return null;
    }

    /**
     * 得到年末
     *
     * @return
     */
    public static Date getYearEndDay(Date inTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(inTime);
        int x = c.get(Calendar.YEAR);
        try {
            return sdf.parse(x + "-12" + "-31");
        } catch (Exception e) {

        }
        return null;
    }

    public static Date getThisHourStartTime(Date time) {
        final LocalDateTime thisHourStart = LocalDateTime.of(DateUtil.date2LocalDateTime(time).getYear(),
                DateUtil.date2LocalDateTime(time).getMonth(),
                DateUtil.date2LocalDateTime(time).getDayOfMonth(),
                DateUtil.date2LocalDateTime(time).getHour(),
                0, 0);
        return DateUtil.localDateTime2Date(thisHourStart);
    }

    public static Date getThisHourEndTime(Date time) {
        final LocalDateTime thisHourEnd = LocalDateTime.of(DateUtil.date2LocalDateTime(time).getYear(),
                DateUtil.date2LocalDateTime(time).getMonth(),
                DateUtil.date2LocalDateTime(time).getDayOfMonth(),
                DateUtil.date2LocalDateTime(time).getHour(),
                59, 59);
        return DateUtil.localDateTime2Date(thisHourEnd);
    }

    /**
     * 获取当天起始时间
     *
     * @return 当天起始时间
     */
    public static Date getCurrentDayStartTime() {
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime start = LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 0, 0, 0);
        ZoneId zone = ZoneId.systemDefault();
        Instant instant = start.atZone(zone).toInstant();
        return Date.from(instant);
    }

    /**
     * 获取当天结束时间
     *
     * @return 当天结束时间
     */
    public static Date getCurrentDayEndTime() {
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime end = LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 23, 59, 59);
        ZoneId zone = ZoneId.systemDefault();
        Instant instant = end.atZone(zone).toInstant();
        return Date.from(instant);
    }


    /**
     * 获取短时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM-dd
     */
    public static String getStringDateShort(Date date) {
        return getStringDateShort(date, "");
    }

    public static String getStringDateShort(Date date, String defaultStr) {
        if (date == null) {
            return defaultStr;
        }
        String dateString = sdf.format(date);
        return dateString;
    }

    /**
     * 获取时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM-dd HH:mm:ss
     */
    public static String getFullDateString(Date date) {
        return getFullDateString(date, "");
    }

    public static String getFullDateString(Date date, String defaultStr) {
        if (date == null) {
            return defaultStr;
        }
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return format.format(date);
    }

    /**
     * 获取时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM
     */
    public static String getMonthString(Date date) {
        return getMonthString(date, "");
    }

    public static String getMonthString(Date date, String defaultStr) {
        if (date == null) {
            return defaultStr;
        }
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
        return format.format(date);
    }

    /**
     * 获取时间字符串
     *
     * @param date    需要转换格式的日期
     * @param pattern 时间字符串格式
     * @return 返回时间字符串
     */
    public static String getFormatString(Date date, String pattern) {
        return getFormatString(date, pattern, "");
    }

    public static String getFormatString(Date date, String pattern, String defaultStr) {
        if (date == null) {
            return defaultStr;
        }
        SimpleDateFormat format = new SimpleDateFormat(pattern);
        return format.format(date);
    }

    /**
     * 获取下月月初时间
     */
    public static Date getNextMonthStartTime() {
        return DateUtil.localDate2Date(LocalDateTime.now()
                .with(TemporalAdjusters.lastDayOfMonth()).plusDays(1L).toLocalDate());
    }

    /**
     * @param offsetYear
     * @return 当前时间 + offsetYear
     */
    public static Timestamp getNowExpiredYear(int offsetYear) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.YEAR, offsetYear);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offset
     * @return 当前时间 + offsetMonth
     */
    public static Timestamp getNowExpiredMonth(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.MONTH, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offset
     * @return 当前时间 + offsetDay
     */
    public static Timestamp getNowExpiredDay(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.DATE, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offset
     * @return 当前时间 + offsetDay
     */
    public static Timestamp getNowExpiredHour(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.HOUR, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offsetSecond
     * @return 当前时间 + offsetSecond
     */
    public static Timestamp getNowExpiredSecond(int offsetSecond) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.SECOND, offsetSecond);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offset
     * @return 当前时间 + offset
     */
    public static Timestamp getNowExpiredMinute(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.MINUTE, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * @param offset
     * @return 指定时间 + offsetDay
     */
    public static Timestamp getExpiredDay(Date givenDate, int offset) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(Calendar.DATE, offset);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 实现ORACLE中ADD_MONTHS函数功能
     *
     * @param offset
     * @return 指定时间 + offsetMonth
     */
    public static Timestamp getExpiredMonth(Date givenDate, int offset) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(Calendar.MONTH, offset);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * @param
     * @return 指定时间 + offsetYear
     */
    public static Timestamp getExpiredYear(Date givenDate, int offsetYear) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(Calendar.YEAR, offsetYear);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * @param second
     * @return 指定时间 + offsetSecond
     */
    public static Timestamp getExpiredSecond(Date givenDate, int second) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(Calendar.SECOND, second);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 计算时间差
     *
     * @param givenDate 日期
     * @param offset    2
     * @param type      日期字段类型
     * @return
     */
    public static Timestamp getExpired(Date givenDate, int offset, Integer type) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(type, offset);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 计算日期差
     *
     * @param d1   开始日期
     * @param d2   截止日期
     * @param type 类型
     * @return 计算结果
     */
    public static Integer sub(Date d1, Date d2, String type) {
        if (d1 == null || d2 == null) {
            return null;
        }
        long result = 0L;
        long DAY = 24 * 60 * 60 * 1000;
        long sub = d1.getTime() - d2.getTime();
        long daysub = (sub / DAY);
        long y1 = d1.getYear(), y2 = d2.getYear();
        long m1 = d1.getMonth(), m2 = d2.getMonth();
        long monthsub = (y1 - y2) * 12 + (m1 - m2);
        //月
        if ("m".equals(type)) {
            result = monthsub;
            //年
        } else if ("y".equals(type)) {
            result = monthsub / 12;
            //天
        } else if ("d".equals(type)) {
            result = daysub;
        }
        return (int) result;
    }

    /**
     * 判断是否是凌晨
     *
     * @param date
     * @return
     */
    public static boolean isBeforeDawn(Date date) {
        if (Objects.isNull(date)) {
            return false;
        }
        LocalDateTime dateTime = date2LocalDateTime(date);
        SimpleDateFormat df = new SimpleDateFormat("HH");
        String str = df.format(localDateTime2Date(dateTime));
        int a = Integer.parseInt(str);
        return a >= 0 && a <= 6;
    }

    /**
     * 判断是否是上午
     *
     * @param date
     * @return
     */
    public static boolean isMorning(Date date) {
        if (Objects.isNull(date)) {
            return false;
        }
        LocalDateTime dateTime = date2LocalDateTime(date);
        SimpleDateFormat df = new SimpleDateFormat("HH");
        String str = df.format(localDateTime2Date(dateTime));
        int a = Integer.parseInt(str);
        return a > 6 && a <= 12;
    }

    /**
     * 判断是否是下午
     *
     * @param date
     * @return
     */
    public static boolean isAfternoon(Date date) {
        if (Objects.isNull(date)) {
            return false;
        }
        LocalDateTime dateTime = date2LocalDateTime(date);
        SimpleDateFormat df = new SimpleDateFormat("HH");
        String str = df.format(localDateTime2Date(dateTime));
        int a = Integer.parseInt(str);
        return a > 12 && a <= 18;
    }

    /**
     * 判断是否是晚上
     *
     * @param date
     * @return
     */
    public static boolean isEvening(Date date) {
        if (Objects.isNull(date)) {
            return false;
        }
        LocalDateTime dateTime = date2LocalDateTime(date);
        SimpleDateFormat df = new SimpleDateFormat("HH");
        String str = df.format(localDateTime2Date(dateTime));
        int a = Integer.parseInt(str);
        return a > 18 && a <= 23;
    }

    public static void main(String[] args) throws ParseException {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = format.parse("2020-09-19 15:45:30");

        Timestamp expired = DateUtil.getExpired(date, 3, Calendar.DATE);
        if (!isBeforeDawn(expired)) {
            expired = getExpired(startDate(expired), 1, Calendar.DATE);
            System.out.println(expired);
        }
        System.out.println(getEndInTime(getMonthEndDay()));
        System.out.println(expired);
        System.out.println(startDate(DateUtil.getMonthFirstDay(date)));
        ;
    }
}
