package cn.fw.valhalla.rpc.sms;

import cn.fw.common.exception.BusinessException;
import cn.fw.data.base.domain.common.Message;
import cn.fw.sms.sdk.api.SmsApiService;
import cn.fw.sms.sdk.api.dto.BizSmsTemplateResult;
import cn.fw.sms.sdk.api.dto.SmsMessageReq;
import cn.fw.sms.sdk.api.enums.TemplateTypeEnum;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : SmsRpcService
 * @description :
 * @date: 2021-03-31 18:28
 */
@Slf4j
@Service
public class SmsRpcService {
    private final SmsApiService smsApiService;

    @Autowired
    public SmsRpcService(final SmsApiService smsApiService) {
        this.smsApiService = smsApiService;
    }

    /**
     * 获取短信模板
     *
     * @param shopId     门店id
     * @param followType 跟进类型
     * @return 短信模板
     */
    public String getSmsTemplate(@NonNull Long shopId, FollowTypeEnum followType) {
        TemplateTypeEnum type = getTemplateType(followType);
        BV.notNull(type, () -> "跟进类型不正确");
        Message<BizSmsTemplateResult> msg = smsApiService.getTemplate(shopId, type);
        if (!msg.isSuccess()) {
            log.error("调用sms系统查询短信模板失败：{}", msg.getResult());
            throw new BusinessException(msg.getResult());
        }
        String content = Optional.ofNullable(msg.getData()).map(BizSmsTemplateResult::getContent).orElse(null);
        BV.isNotBlank(content, () -> String.format("门店[%s]没有有效的短信模板", shopId));
        return content;
    }

    /**
     * 发送模板短信
     *
     * @param templateCode
     * @param phone
     * @param content
     * @param shopId
     */
    public void sendSms(String templateCode, String phone, Map<String, Object> content, Long shopId) {
        if (StringUtils.isEmpty(templateCode)) {
            log.error("调用sms系统发送短信失败：模板不能为空");
            return;
        }
        if (StringUtils.isEmpty(phone)) {
            log.error("调用sms系统发送短信失败：电话号码不能为空");
            return;
        }
        SmsMessageReq req = new SmsMessageReq();
        req.setPhoneNumbers(phone);
        req.setShopId(shopId);
        req.setTemplateCode(templateCode);
        req.setContent(content);
        Message<Boolean> msg = smsApiService.send(req);
        if (!msg.isSuccess()) {
            log.error("调用sms系统发送短信失败：{}", msg.getResult());
        }
    }

    private TemplateTypeEnum getTemplateType(FollowTypeEnum type) {
        switch (type) {
            case AC:
                return TemplateTypeEnum.AC;
            case FM:
                return TemplateTypeEnum.FM;
            case IR:
                return TemplateTypeEnum.IR;
            case RM:
                return TemplateTypeEnum.RM;
            default:
                return null;
        }
    }
}
