package cn.fw.valhalla.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.CustomerChangeDto;
import cn.fw.valhalla.domain.query.CustomerQueryVO;
import cn.fw.valhalla.domain.vo.customer.*;
import cn.fw.valhalla.service.bus.CommonService;
import cn.fw.valhalla.service.bus.cust.ContactBizService;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.bus.cust.CustomerChangeBizService;
import cn.fw.valhalla.service.bus.cust.PickUpCustomerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @className : CustomerController
 * @description : 档案控制器
 * @date: 2020-08-12 15:30
 */
@Slf4j
@Validated
@RestController
@Authorization(AuthType.APP)
@RequestMapping("/app/customer")
public class CustomerController {
    private final CustomerBizService customerBizService;
    private final PickUpCustomerService pickUpCustomerService;
    private final CustomerChangeBizService customerChangeBiz;
    private final ContactBizService contactBizService;
    private final CommonService commonService;


    @Autowired
    public CustomerController(final CustomerBizService customerBizService,
                              final PickUpCustomerService pickUpCustomerService,
                              final CustomerChangeBizService customerChangeBiz,
                              final ContactBizService contactBizService,
                              final CommonService commonService) {
        this.customerBizService = customerBizService;
        this.pickUpCustomerService = pickUpCustomerService;
        this.customerChangeBiz = customerChangeBiz;
        this.contactBizService = contactBizService;
        this.commonService = commonService;
    }

    /**
     * 分页查询客户信息列表 （管理人员）
     *
     * @param queryVO 查询条件
     */
    @GetMapping("/page")
    @IgnoreAuth
    @ControllerMethod("查询客户列表(管理人员纬度)")
    public Message<AppPage<CustomerListVO>> customerList(final CustomerQueryVO queryVO) {
        AppPage<CustomerListVO> page = customerBizService.getList(queryVO);
        return success(page);
    }

    /**
     * 分页查询客户信息列表 （服务顾问自己的）
     *
     * @param currentUser
     * @param queryVO
     * @return
     */
    @GetMapping("/page/own")
    @ControllerMethod("查询客户列表(自己的)")
    public Message<AppPage<CustomerListVO>> customerOwnList(@CurrentUser LoginAuthBean currentUser,
                                                            final CustomerQueryVO queryVO) {
        queryVO.setConsultantId(currentUser.getUserId());
        AppPage<CustomerListVO> page = customerBizService.getList(queryVO);
        return success(page);
    }

    /**
     * 根据ID查询档案信息
     *
     * @param cusId 档案ID
     */
    @GetMapping("/detailById")
    @IgnoreAuth
    @ControllerMethod("根据ID查询档案信息")
    public Message<CustomerDetailVO> getDetailById(@NotNull(message = "档案ID不能为空") final Long cusId) {
        return success(customerBizService.getDetailById(cusId));
    }


    /**
     * 根据车牌号查询档案信息
     *
     * @param plateNo 车牌号
     */
    @GetMapping("/detailByPlate")
    @IgnoreAuth
    @ControllerMethod("根据车牌号查询档案信息")
    public Message<PickUpCustomerDetailVO> getDetailByPlate(@CurrentUser LoginAuthBean currentUser, @NotBlank(message = "车牌号不能为空") final String plateNo) {
        return success(pickUpCustomerService.getDetailByPlateNo(currentUser, plateNo));
    }

    /**
     * 通过智能通话拨打电话
     *
     * @param currentUser
     * @param id
     * @param accidentCar
     * @return
     */
    @GetMapping("/call")
    @ControllerMethod("通过智能通话拨打电话")
    public Message<String> call(@CurrentUser LoginAuthBean currentUser,
                                @NotNull(message = "档案ID不能为空") final Long id,
                                final Boolean accidentCar
    ) {
        return success(commonService.call(currentUser.getUserId(), id, accidentCar));
    }

    /**
     * 根据车架号查询档案信息
     * 如果发现档案是临牌则修改车牌为现在的
     *
     * @param frameNo 车架号
     */
    @GetMapping("/detailByFrameNo")
    @ControllerMethod("根据车架号号查询档案信息")
    public Message<PickUpCustomerDetailVO> getDetailByFrameNo(@CurrentUser LoginAuthBean currentUser, @NotBlank(message = "车架号不能为空") final String frameNo) {
        return success(pickUpCustomerService.getDetailByFrameNo(currentUser, frameNo));
    }

    /**
     * 根据车架号修改档案车牌号
     *
     * @param frameNo
     * @param plateNo
     * @return
     */
    @PostMapping("/fixPlateNo")
    @ControllerMethod("根据车架号修改档案车牌号")
    public Message<Long> fixPlateNo(@CurrentUser LoginAuthBean currentUser, @NotBlank(message = "车架号不能为空") final String frameNo, @NotBlank(message = "车牌号不能为空") final String plateNo) {
        return success(pickUpCustomerService.fixPlateNo(frameNo, plateNo, currentUser));
    }

    /**
     * 根据ID查询客户信息
     *
     * @param cusId 客户ID
     */
    @GetMapping("/vehicle")
    @ControllerMethod("根据ID查询车辆信息")
    public Message<VehicleRecordVO> getCarInfoById(@NotNull(message = "档案ID不能为空") final Long cusId) {
        return success(customerBizService.getCarInfoById(cusId));
    }

    /**
     * 根据ID查询客户信息
     *
     * @param cusId 客户ID
     */
    @GetMapping("/owner")
    @ControllerMethod("根据ID查询所有人信息")
    public Message<OwnerVO> getOwnerInfo(@NotNull(message = "档案ID不能为空") final Long cusId) {
        return success(customerBizService.getOwnerInfo(cusId));
    }

    /**
     * 根据ID查询客户信息
     *
     * @param cusId 客户ID
     */
    @GetMapping("/getMore")
    @ControllerMethod("根据ID全部档案信息")
    public Message<List<MoreVehicleVO>> getMore(@NotNull(message = "客户ID不能为空") final Long cusId, @CurrentUser LoginAuthBean currentUser) {
        return success(customerBizService.getMoreArchive(cusId, currentUser.getGroupId()));
    }


    /**
     * 修改档案标签
     *
     * @param tags 标签枚举
     */
    @PostMapping("/updateTagById")
    @ControllerMethod("修改档案标签")
    public Message<Integer> updateTagById(@NotBlank(message = "标签不能为空") final String tags,
                                          @NotNull(message = "id不能为空") final Long cusId) {
        return success(customerChangeBiz.updateTagById(cusId, tags));
    }


    /**
     * 查询送修人、同行人等信息
     *
     * @param cusId 档案id
     */
    @GetMapping("/contact/list")
    @ControllerMethod("查询送修人、同行人等信息")
    public Message<List<CustomerContactVO>> contactList(@NotNull(message = "档案ID不能为空") final Long cusId) {
        return success(contactBizService.contactList(cusId));
    }

    /**
     * 获取档案变更二维码
     *
     * @param currentUser
     * @param customerChangeDto
     * @return
     */
    @PostMapping("/change/qrcode")
    @ControllerMethod("获取档案变更二维码")
    public Message<CustomerChangeQrCodeVO> getQrCode(@CurrentUser LoginAuthBean currentUser,
                                                     @Validated @RequestBody CustomerChangeDto customerChangeDto) {
        return success(customerChangeBiz.createQrCode(currentUser, customerChangeDto));
    }

    /**
     * 主动放弃
     *
     * @param cusId
     */
    @PostMapping("/abandon")
    @ControllerMethod("主动放弃客户")
    public Message<Void> abandon(@NotNull(message = "id不能为空") final Long cusId, final String reason) {
        customerBizService.abandon(cusId, reason);
        return success();
    }

    /**
     * 继续跟进
     *
     * @param cusId
     */
    @PostMapping("/continue")
    @ControllerMethod("继续跟进客户")
    public Message<Void> continueFollow(@CurrentUser LoginAuthBean currentUser, @NotNull(message = "id不能为空") final Long cusId) {
        customerBizService.continueFollow(currentUser, cusId);
        return success();
    }
}
