package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.db.pool.PublicPool;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.enums.DefeatReasonEnum;
import cn.fw.valhalla.domain.enums.PublicClueStateEnum;
import cn.fw.valhalla.domain.enums.PublicPoolTypeEnum;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.data.PublicCluePoolService;
import cn.fw.valhalla.service.data.PublicPoolService;
import cn.fw.valhalla.service.data.StammkundePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : PublicClueTask
 * @description : 公共客户线索池处理任务
 * @date: 2021-03-30 17:24
 */
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
@Slf4j
public class PublicClueTask {
    private final PublicCluePoolService publicCluePoolService;
    private final PublicPoolService publicPoolService;
    private final CustomerBizService customerBizService;
    private final StammkundePoolService stammkundePoolService;

    @Autowired
    public PublicClueTask(final PublicCluePoolService publicCluePoolService,
                          final PublicPoolService publicPoolService,
                          final CustomerBizService customerBizService,
                          final StammkundePoolService stammkundePoolService) {
        this.publicCluePoolService = publicCluePoolService;
        this.publicPoolService = publicPoolService;
        this.customerBizService = customerBizService;
        this.stammkundePoolService = stammkundePoolService;
    }


    /**
     * 开始任务
     * 每天执行两次
     */
    @Scheduled(cron = "0/30 * * * * *")
    public void endClue() {
        try {
            List<PublicCluePool> list = publicCluePoolService.list(Wrappers.<PublicCluePool>lambdaQuery()
                    .eq(PublicCluePool::getState, PublicClueStateEnum.ONGOING)
                    .le(PublicCluePool::getDeadline, DateUtil.startDate(new Date()))
                    .last("limit 0, 1000")
            );
            if (CollectionUtils.isEmpty(list)) {
                return;
            }
            closeData(list);
        } catch (Exception e) {
            log.error("定时处理公共客户线索池失败", e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void closeData(List<PublicCluePool> list) {
        List<PublicPool> publicPoolList = new ArrayList<>();
        List<Long> customerIdList = new ArrayList<>();

        for (PublicCluePool cluePool : list) {
            cluePool.setCloseTime(new Date());
            cluePool.setState(PublicClueStateEnum.DEFEAT);

            Long customerId = cluePool.getCustomerId();
            CustomerDetailDto detailDto = customerBizService.queryById(customerId);
            if (Objects.isNull(detailDto)) {
                continue;
            }
            detailDto.setAdviserId(cluePool.getAdviserId());
            detailDto.setAdviserName(cluePool.getAdviserName());
            detailDto.setShopId(cluePool.getShopId());
            PublicPool publicPool = customerBizService.createPublicPool(detailDto, PublicPoolTypeEnum.PD, PublicPoolTypeEnum.PD.getName());
            publicPoolList.add(publicPool);
            customerIdList.add(detailDto.getId());
            stammkundePoolService.reject(cluePool.getCustomerId(), cluePool.getGroupId(), DefeatReasonEnum.LC);
        }

        if (!CollectionUtils.isEmpty(publicPoolList)) {
            publicPoolService.saveBatch(publicPoolList);
        }

        customerBizService.move2PublicPool(customerIdList);
        publicCluePoolService.updateBatchById(list);
    }
}
