package cn.fw.valhalla.service.bus;

import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.db.customer.CustomerImport;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.member.MemberRpcService;
import cn.fw.valhalla.rpc.member.dto.MemberUserDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.sdk.enums.GenderEnum;
import cn.fw.valhalla.service.data.CustomerBaseInfoService;
import cn.fw.valhalla.service.data.CustomerImportService;
import cn.fw.valhalla.service.data.CustomerService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : CustomerImportBizService
 * @description : 客户档案导入处理
 * @date: 2020-11-18 15:17
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class CustomerImportBizService {
    private final CustomerBaseInfoService customerBaseInfoService;
    private final CustomerService customerService;
    private final CustomerImportService customerImportService;
    private final OopService oopService;
    private final UserService userService;
    private final MemberRpcService memberRpcService;

    private final static Long GROUPID = 2L;
    private static final String PHONE_NUMBER_REG = "^((\\+86)|(86))?(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\\d{8}$";

    public void integrated() {
        do {
            dealCustomer(query());
        } while (canDo());
    }

    public void apportion() {
        List<ShopDTO> allShops = oopService.getAllShops(GROUPID);
        for (ShopDTO shop : allShops) {
            List<PostUserDTO> userDTOS = userService.getUserByRole(shop.getId(), RoleCode.FWGW);
            apportionCustomer(userDTOS, shop.getId());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void apportionCustomer(List<PostUserDTO> userDTOS, Long shopId) {
        if (CollectionUtils.isEmpty(userDTOS)) {
            log.error("门店：{}没有配置服务顾问", shopId);
            return;
        }
        List<Long> userIds = userDTOS.stream().map(PostUserDTO::getUserId).collect(Collectors.toList());
        int i = customerNum(userIds) + customerNum(shopId);
        int average = i / userDTOS.size();
        if (i % userDTOS.size() != 0) {
            average = average + 1;
        }

        for (PostUserDTO userDTO : userDTOS) {
            int i1 = personNum(userDTO.getUserId());
            int diff = average - i1;
            if (diff <= 0) {
                continue;
            }
            List<CustomerImport> imports = customerImportService.list(Wrappers.<CustomerImport>lambdaQuery()
                    .eq(CustomerImport::getShopId, shopId)
                    .eq(CustomerImport::getNeedAssigning, Boolean.TRUE)
                    .last(" limit " + diff)
            );
            if (CollectionUtils.isEmpty(imports)) {
                continue;
            }
            for (CustomerImport anImport : imports) {
                anImport.setAdviserId(userDTO.getUserId());
                anImport.setAdviserName(userDTO.getUserName());
                anImport.setNeedAssigning(Boolean.FALSE);
            }
            customerImportService.updateBatchById(imports);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    public void dealCustomer(List<CustomerImport> importList) {
        for (CustomerImport customerImport : importList) {
            String frameNo = customerImport.getFrameNo();
            Customer customer = customerService.queryByFrameNo(frameNo, GROUPID);
            if (Objects.isNull(customer)) {
                customer = queryByVin(frameNo);
            }
            if (Objects.isNull(customer)) {
                createCustomer(customerImport);
            } else {
                updateCustomer(customerImport, customer);
            }
            if (Objects.isNull(customerImport.getAdviserId())) {
                customerImport.setNeedAssigning(Boolean.TRUE);
            }
            if (1 != customerImport.getMark()) {
                customerImport.setMark(-1);
            }
        }
        customerImportService.updateBatchById(importList);
    }

    private void updateCustomer(CustomerImport customerImport, Customer customer) {
        String mobile = customerImport.getMobile();
        String name = customerImport.getName();
        CustomerBaseInfo baseInfo = customerBaseInfoService.getById(customer.getBaseId());
        if (Objects.isNull(baseInfo)) {
            baseInfo = customerBaseInfoService.queryByMobile(customerImport.getMobile(), GROUPID);
        }
        if (Objects.isNull(baseInfo)) {
            baseInfo = createBaseInfo(mobile, name);
        }
        if (Objects.isNull(baseInfo)) {
            return;
        }
        customer.setBaseId(baseInfo.getId());
        customer.setPlateNo(customerImport.getPlateNo());
        customer.setFrameNo(customerImport.getFrameNo());
        customer.setSpecCode(StringUtils.isValid(customerImport.getSpecCode()) ? customerImport.getSpecCode() : null);
        customer.setEngineNo(StringUtils.isValid(customerImport.getEngineNo()) ? customerImport.getEngineNo() : null);
        customer.setCurrentMileage(customerImport.getCurrentMileage());
        customer.setArrivalTime(customerImport.getArrivalTime());
        customer.setShopId(customerImport.getShopId());
        customer.setAdviserId(customerImport.getAdviserId());
        customer.setYn(Boolean.TRUE);
        customerService.updateById(customer);
        customerImport.setMark(1);
        customerImport.setOldId(customer.getId());
    }

    private void createCustomer(CustomerImport customerImport) {
        String mobile = customerImport.getMobile();
        if (StringUtils.isEmpty(mobile) || !Pattern.matches(PHONE_NUMBER_REG, mobile)) {
            return;
        }
        CustomerBaseInfo baseInfo = customerBaseInfoService.queryByMobile(mobile, GROUPID);
        if (Objects.isNull(baseInfo)) {
            baseInfo = createBaseInfo(customerImport.getMobile(), customerImport.getName());
        }
        if (Objects.isNull(baseInfo)) {
            return;
        }
        Customer customer = new Customer();
        customer.setBaseId(baseInfo.getId());
        customer.setPlateNo(customerImport.getPlateNo());
        customer.setFrameNo(customerImport.getFrameNo());
        customer.setSpecCode(StringUtils.isValid(customerImport.getSpecCode()) ? customerImport.getSpecCode() : null);
        customer.setEngineNo(StringUtils.isValid(customerImport.getEngineNo()) ? customerImport.getEngineNo() : null);
        customer.setCurrentMileage(customerImport.getCurrentMileage());
        customer.setArrivalTime(customerImport.getArrivalTime());
        customer.setShopId(customerImport.getShopId());
        customer.setAdviserId(customerImport.getAdviserId());
        customer.setGroupId(GROUPID);
        customer.setYn(Boolean.TRUE);
        customerService.saveOrUpdate(customer);
        customerImport.setMark(1);
        customerImport.setOldId(customer.getId());
    }

    private CustomerBaseInfo createBaseInfo(String mobile, String name) {
        MemberUserDTO memberUserDTO = memberRpcService.queryByMobile(mobile);
        if (Objects.isNull(memberUserDTO)) {
            MemberUserDTO dto = new MemberUserDTO();
            dto.setCardNo(mobile);
            dto.setPhone(mobile);
            dto.setRealName(name);
            dto.setNickName(name);
            memberUserDTO = memberRpcService.register(dto);
        }
        if (Objects.isNull(memberUserDTO)) {
            return null;
        }
        CustomerBaseInfo info = new CustomerBaseInfo();
        info.setName(memberUserDTO.getRealName());
        info.setMemberId(memberUserDTO.getUserId());
        info.setContact(memberUserDTO.getRealName());
        info.setMobile(memberUserDTO.getPhone());
        info.setGender(Objects.isNull(GenderEnum.ofValue(memberUserDTO.getSex())) ? GenderEnum.UNKNOWN : GenderEnum.ofValue(memberUserDTO.getSex()));
        info.setIdCode(memberUserDTO.getCredentNo());
        info.setGroupId(GROUPID);
        info.setYn(Boolean.TRUE);
        customerBaseInfoService.save(info);
        return info;
    }

    private Customer queryByVin(String frameNo) {
        return customerService.getOne(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getFrameNo, frameNo)
                .eq(Customer::getGroupId, GROUPID)
                .orderByDesc(Customer::getUpdateTime)
                .last("limit 1"));
    }

    private List<CustomerImport> query() {
        List<CustomerImport> list = customerImportService.list(Wrappers.<CustomerImport>lambdaQuery()
                .eq(CustomerImport::getMark, 0)
                .last(" limit 1000")
        );
        return Optional.ofNullable(list).orElse(new ArrayList<>());
    }

    private boolean canDo() {
        return customerImportService.count(Wrappers.<CustomerImport>lambdaQuery()
                .eq(CustomerImport::getMark, Boolean.FALSE)
        ) > 0;
    }

    private int personNum(Long userId) {
        return customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getYn, Boolean.TRUE)
        );
    }

    private int customerNum(List<Long> ids) {
        return customerService.count(Wrappers.<Customer>lambdaQuery()
                .in(Customer::getAdviserId, ids)
                .eq(Customer::getYn, Boolean.TRUE)
        );
    }

    private int customerNum(Long shopId) {
        return customerImportService.count(Wrappers.<CustomerImport>lambdaQuery()
                .eq(CustomerImport::getShopId, shopId)
                .eq(CustomerImport::getNeedAssigning, Boolean.TRUE)
        );
    }
}
