package cn.fw.valhalla.service.bus.cust;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.PassportAuthBean;
import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.sdk.api.para.BusinessType;
import cn.fw.hermes.sdk.api.para.MsgParamCondition;
import cn.fw.passport.sdk.api.param.WxBCodeParam;
import cn.fw.starter.redis.redis.RedisUtil;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.MessageFormatUtil;
import cn.fw.valhalla.common.utils.QRCodeKit;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.AffiliationRecord;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.db.follow.FollowRecordLog;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.dto.CustomerChangeDto;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.dto.StammkundeDto;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.vo.customer.CustomerChangeInfoVO;
import cn.fw.valhalla.domain.vo.customer.CustomerChangeQrCodeVO;
import cn.fw.valhalla.domain.vo.follow.FollowDetailVO;
import cn.fw.valhalla.rpc.angel.dto.InsuranceDTO;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.member.MemberRpcService;
import cn.fw.valhalla.rpc.member.dto.MemberUserDTO;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.rpc.passport.PassportService;
import cn.fw.valhalla.sdk.enums.CarUseTypeEnum;
import cn.fw.valhalla.sdk.enums.GenderEnum;
import cn.fw.valhalla.sdk.enums.ReasonEnum;
import cn.fw.valhalla.sdk.param.ChangeAdviserReq;
import cn.fw.valhalla.service.data.*;
import cn.fw.valhalla.service.event.CustomerDefeatedEvent;
import cn.fw.valhalla.service.event.PublicPoolEvent;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : CustomerChangeBizService
 * @description : 档案变更服务
 * @date: 2020-08-12 11:23
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CustomerChangeBizService extends AbstractCustomerService {
    public final String CUSTOMER_CHANGE_CODE = "valhalla:customer:change:key:{0}:{1}";

    private final RedisUtil redisUtil;
    /**
     * 会员服务
     */
    private final PassportService passportService;
    private final MemberRpcService memberRpcService;
    private final FollowTaskService followTaskService;
    private final CustomerCluePoolService customerCluePoolService;
    private final AffiliationRecordService affiliationRecordService;
    private final PublicPoolService publicPoolService;
    private final ApplicationEventPublisher eventPublisher;
    private final FollowRecordService followRecordService;
    private final FollowNoticeRecordService followNoticeRecordService;
    private final FollowRecordLogService followRecordLogService;

    /**
     * 生成档案变更二维码
     *
     * @param user
     * @param customerChangeDto
     * @return
     */
    public CustomerChangeQrCodeVO createQrCode(LoginAuthBean user, CustomerChangeDto customerChangeDto) {
        String plateNo = customerChangeDto.getPlateNo();
        String vin = customerChangeDto.getFrameNo();
        ShopDTO shopDTO = queryDealId(user.getUserId(), RoleCode.FWGW);
        BV.notNull(shopDTO, () -> "用户权限不正确");

        if (customerChangeDto.getId() == null) {
            customerChangeDto.setGroupId(shopDTO.getGroupId());
            customerChangeDto.setDealerId(shopDTO.getDealerId());
            customerChangeDto.setShopId(shopDTO.getId());
            customerChangeDto.setAdviserId(user.getUserId());
        } else {
            Customer cust = customerService.queryById(customerChangeDto.getId());
            customerChangeDto.setGroupId(cust.getGroupId());
            customerChangeDto.setDealerId(shopDTO.getDealerId());
            customerChangeDto.setShopId(cust.getShopId());
            customerChangeDto.setAdviserId(cust.getAdviserId());
            customerChangeDto.setModelCode(cust.getSpecCode());
        }
        String str = plateNo + vin + user.getUserId();
        if (Objects.isNull(customerChangeDto.getShopId())) {
            customerChangeDto.setShopId(shopDTO.getId());
            customerChangeDto.setAdviserId(user.getUserId());
        }
        String key = MessageFormatUtil.MessageFormatTransfer(CUSTOMER_CHANGE_CODE, customerChangeDto.getShopId(), str);
        try {
            String customerString = JSON.toJSONString(customerChangeDto);
            boolean flag = redisUtil.lock(key, 1000 * 15);
            BV.isTrue(flag, "重复操作，请稍后重试");
            int expires = 60 * 30;
            String s = redisUtil.STRINGS.setEx(key, expires, customerString);
            BV.isTrue(StringUtils.isValid(s), "操作失败，请稍后重试");
            String code = createCode(key, expires, customerChangeDto.getDealerId());
            return new CustomerChangeQrCodeVO(code, key);
        } finally {
            redisUtil.unlock(key);
        }
    }

    /**
     * 根据主键查询档案信息
     *
     * @param customerId
     * @return
     */
    public CustomerChangeInfoVO mpGetById(Long customerId) {
        final CustomerChangeInfoVO detailVO = new CustomerChangeInfoVO();
        Customer customer = customerService.queryById(customerId);
        if (customer == null) {
            return null;
        }
        CustomerDetailDto customerDetailDto = renderDto(customer);
        BeanUtils.copyProperties(customerDetailDto, detailVO);
        detailVO.setId(customer.getId());
        Optional<InsuranceDTO> dtoOptional = queryInsuInfo(customerId);
        dtoOptional.ifPresent(t -> {
            detailVO.setInsurCompanyId(t.getInsurerId());
            detailVO.setInsurCompanyName(t.getInsurerName());
            detailVO.setInsurExpireDate(t.getExpiryDate());
        });
        return detailVO;
    }


    /**
     * 用户维护的基础信息查询
     *
     * @param user
     * @param key
     * @return
     */
    public CustomerChangeInfoVO getInfo(PassportAuthBean user, String key) {
        String s = redisUtil.STRINGS.get(key);
        BV.isTrue(StringUtils.isValid(s), "二维码已失效");
        CustomerChangeDto dto = JSON.parseObject(s, CustomerChangeDto.class);
        final CustomerChangeInfoVO vo = new CustomerChangeInfoVO();
        BeanUtils.copyProperties(dto, vo);
        MemberUserDTO userDto = memberRpcService.user(user.getUserId());
        if (userDto != null) {
            vo.setMobile(userDto.getPhone());
        }
        this.sendMsg(vo.getAdviserId(), false, key);
        return vo;
    }

    /**
     * 档案修改
     *
     * @param saveDto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Long updateOrSave(PassportAuthBean user, CustomerChangeDto saveDto) {
        final String key = saveDto.getKey();
        final Long adviserId = saveDto.getAdviserId();
        saveDto.setMemberId(user.getUserId());
        Long nid;

        CustomerBaseInfo baseInfo = new CustomerBaseInfo();
        BeanUtils.copyProperties(saveDto, baseInfo);
        baseInfo.setId(null);
        baseInfo.setGender(GenderEnum.ofValue(saveDto.getGender()));
        baseInfo.setUpdateTime(new Date());

        //客户主动修改信息
        if (StringUtils.isEmpty(key)) {
            Customer customer = customerService.queryById(saveDto.getId());
            baseInfo.setId(customer.getBaseId());
            customerBaseInfoService.updateById(baseInfo);
            return customer.getId();
        }

        //车牌号有档案
        if (Objects.nonNull(saveDto.getMotoId())) {
            //vin有档案
            if (Objects.nonNull(saveDto.getId())) {
                //档案和行驶证不一致 作废旧档案 更新档案
                boolean bool = saveDto.getId().equals(saveDto.getMotoId());
                if (!bool) {
                    stammkundePoolService.reject(saveDto.getId(), saveDto.getGroupId(), DefeatReasonEnum.CI);
                    publicCluePoolService.removeClue(saveDto.getId(), adviserId);
                    customerService.forbidden(saveDto.getMotoId());
                    PublicPoolEvent poolEvent = new PublicPoolEvent(saveDto.getMotoId(), saveDto.getGroupId(), true);
                    eventPublisher.publishEvent(poolEvent);
                }
                Customer customer = customerService.queryById(saveDto.getId());
                BV.notNull(customer, "档案信息异常");
                baseInfo.setId(customer.getBaseId());
                customerBaseInfoService.updateById(baseInfo);
                if (Objects.isNull(customer.getAdviserId())) {
                    saveDto.setShopId(null);
                    saveDto.setAdviserId(null);
                }
                BeanUtils.copyProperties(saveDto, customer);
                customer.setUseType(CarUseTypeEnum.ofValue(saveDto.getUseType()));
                customer.setUpdateTime(new Date());
                customer.setYn(Boolean.TRUE);
                nid = this.edit(customer);
            } else {
                stammkundePoolService.reject(saveDto.getId(), saveDto.getGroupId(), DefeatReasonEnum.CI);
                publicCluePoolService.removeClue(saveDto.getId(), adviserId);
                customerService.forbidden(saveDto.getMotoId());
                PublicPoolEvent poolEvent = new PublicPoolEvent(saveDto.getMotoId(), saveDto.getGroupId(), true);
                eventPublisher.publishEvent(poolEvent);
                Customer customer = new Customer();
                BeanUtils.copyProperties(saveDto, customer);
                customer.setUseType(CarUseTypeEnum.ofValue(saveDto.getUseType()));
                customer.setSpecCode(saveDto.getModelCode());
                nid = create(customer, baseInfo);
            }
        } else {
            //vin有档案 行驶证车主和vin档案车主一致的情况app直接替换车牌不会到这个流程
            if (Objects.nonNull(saveDto.getId())) {
                stammkundePoolService.reject(saveDto.getId(), saveDto.getGroupId(), DefeatReasonEnum.CI);
                publicCluePoolService.removeClue(saveDto.getId(), adviserId);
                customerService.forbidden(saveDto.getId());
                PublicPoolEvent poolEvent = new PublicPoolEvent(saveDto.getMotoId(), saveDto.getGroupId(), true);
                eventPublisher.publishEvent(poolEvent);
                saveDto.setId(null);
            }
            Customer customer = new Customer();
            BeanUtils.copyProperties(saveDto, customer);
            customer.setUseType(CarUseTypeEnum.ofValue(saveDto.getUseType()));
            customer.setSpecCode(saveDto.getModelCode());
            nid = create(customer, baseInfo);
            if (Objects.nonNull(saveDto.getId())) {
                changeTaskFollower(saveDto.getMotoId(), customer.getId(), customer.getGroupId());
            }
        }

        if (StringUtils.isValid(key)) {
            redisUtil.KEYS.del(key);
            this.sendMsg(adviserId, true, key);
        }
        return nid;
    }

    /**
     * 变更档案标签
     *
     * @param cusId
     * @param tags
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public int updateTagById(Long cusId, String tags) {
        Customer customer = customerService.queryById(cusId);
        BV.notNull(customer, "档案信息异常");
        customer.setTags(tags);
        customer.setUpdateTime(new Date());
        int updateCount = customerService.getBaseMapper().updateById(customer);
        if (updateCount > 0) {
            customerChangeHandler(customer);
        }
        return updateCount;
    }

    /**
     * 更新保有客专属服务顾问
     *
     * @param changeAdviserReq 新的服务顾问
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean changeAdviser(ChangeAdviserReq changeAdviserReq) {
        Customer customer = customerService.queryById(changeAdviserReq.getCusId());
        if (Objects.isNull(customer)) {
            return true;
        }
        boolean isCard = ReasonEnum.MC.getValue().equals(changeAdviserReq.getReason()) || ReasonEnum.WC.getValue().equals(changeAdviserReq.getReason());
        if (isCard) {
            return true;
        }
        Long adviserId = changeAdviserReq.getAdviserId();
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(adviserId, RoleCode.FWGW);
        if (CollectionUtils.isEmpty(dataRange) || Objects.isNull(dataRange.get(0).getRangeValue())) {
            return true;
        }
        if (adviserId.equals(customer.getAdviserId())) {
            StammkundePool stammkundePool = stammkundePoolService.queryAktivAble(customer.getId(), customer.getGroupId(), customer.getShopId());
            if (Objects.nonNull(stammkundePool)) {
                stammkundePool.setAktiv(Boolean.TRUE);
                stammkundePool.setActivationTime(new Date());
                stammkundePool.setPoolStatus(StammkundeStatusEnum.KUNDE);
                Date createTime = DateUtil.getBeginInTime(stammkundePool.getCreateTime());
                Date expiration = DateUtil.getBeginInTime(new Date());

                boolean flag = expiration.after(createTime);

                if (StammkundeSourcesEnum.PUBLIC_POOL.equals(stammkundePool.getSources())) {
                    PublicCluePool cluePool = publicCluePoolService.queryByAdviserAndCust(customer.getId(), adviserId);
                    if (Objects.nonNull(cluePool)) {
                        flag = flag || expiration.after(cluePool.getCreateTime());
                        List<FollowRecord> recordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                                .eq(FollowRecord::getTaskId, cluePool.getId())
                                .eq(FollowRecord::getType, FollowTypeEnum.OT)
                                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                                .orderByAsc(FollowRecord::getId)
                        );
                        if (!CollectionUtils.isEmpty(recordList)) {
                            FollowRecord followRecord = recordList.get(0);
                            int count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery().eq(FollowRecordLog::getRecordId, followRecord.getId()));
                            flag = flag && (count > 0);
                        }
                    }

                    if (flag) {
                        publicCluePoolService.closeClue(customer.getId(), adviserId);
                    } else {
                        stammkundePool.setCreateTime(new Date());
                        stammkundePool.setSources(StammkundeSourcesEnum.NATURAL);
                        publicCluePoolService.removeClue(customer.getId(), adviserId);
                    }
                }
                stammkundePoolService.updateById(stammkundePool);
            }
            return true;
        }
        Long shopId = dataRange.get(0).getRangeValue();

        boolean updated = customerService.update(Wrappers.<Customer>lambdaUpdate()
                .set(Customer::getAdviserId, adviserId)
                .set(Customer::getShopId, shopId)
                .eq(Customer::getId, changeAdviserReq.getCusId()));

        BV.isTrue(updated, () -> "变更档案服务顾问失败，请重试");

        if (Objects.nonNull(customer.getAdviserId()) && Objects.nonNull(customer.getShopId())) {
            Integer reason = changeAdviserReq.getReason();
            DefeatReasonEnum reasonEnum = DefeatReasonEnum.ofValue(reason);
            BV.notNull(reasonEnum, () -> "档案变更原因不正确");
            AffiliationRecord entity = createEntity(customer, adviserId, shopId);
            entity.setReason(reasonEnum.getName());
            affiliationRecordService.save(entity);
            //处理保有客池数据
            StammkundeDto stammkundeDto = new StammkundeDto();
            stammkundeDto.setCustomerId(customer.getId());
            stammkundeDto.setReason(reasonEnum);
            stammkundeDto.setShopId(customer.getShopId());
            UserInfoDTO user = userService.user(adviserId);
            stammkundeDto.setNewUserId(adviserId);
            if (Objects.nonNull(user)) {
                stammkundeDto.setNewUserName(user.getUserName());
            }
            ShopDTO shop = oopService.shop(shopId);
            stammkundeDto.setNewShopId(shopId);
            if (Objects.nonNull(shop)) {
                stammkundeDto.setNewShopName(shop.getShortName());
            }
            stammkundeDto.setGroupId(customer.getGroupId());
            stammkundePoolService.reject(stammkundeDto);
            publicCluePoolService.removeClue(customer.getId(), adviserId);
            create(customer, adviserId, shopId);
        }

        if (Objects.isNull(customer.getAdviserId()) && Objects.isNull(customer.getShopId())) {
            publicPoolService.removeByCustomerId(customer.getId(), customer.getGroupId());
            create(customer, adviserId, shopId);
        }

        return updated;
    }

    /**
     * 二次分配更换跟进人
     *
     * @param task
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public StammkundePool changeFollowUser(FollowTask task) throws BusinessException {
        Customer customer = customerService.getById(task.getCustomerId());
        if (Objects.isNull(customer)) {
            log.info("档案信息异常，custId: [{}]", task.getCustomerId());
        }
        BV.notNull(customer, () -> "档案信息不存在");
        List<PostUserDTO> userList = userService.getUserByRole(customer.getShopId(), RoleCode.FWGW);

        BV.isNotEmpty(userList, () -> String.format("更换跟进人员失败：门店[%s]没有服务顾问", customer.getShopId()));
        Collections.shuffle(userList);
        PostUserDTO userDTO = userList.stream().filter(u -> !u.getUserId().equals(task.getFollowUser())).findAny().orElse(null);
        if (Objects.isNull(userDTO)) {
            return null;
        }
        AffiliationRecord entity = createEntity(customer, userDTO.getUserId(), customer.getShopId());
        customer.setAdviserId(userDTO.getUserId());
        customerService.updateById(customer);
        DefeatReasonEnum reasonEnum = DefeatReasonEnum.OT;
        if (FollowTypeEnum.FM.equals(task.getType())) {
            reasonEnum = DefeatReasonEnum.FM;
        }
        if (FollowTypeEnum.RM.equals(task.getType())) {
            reasonEnum = DefeatReasonEnum.RM;
        }
        entity.setReason(reasonEnum.getName());
        affiliationRecordService.save(entity);
        StammkundeDto stammkundeDto = new StammkundeDto();
        stammkundeDto.setCustomerId(customer.getId());
        stammkundeDto.setReason(reasonEnum);
        stammkundeDto.setShopId(customer.getShopId());
        stammkundeDto.setNewUserId(userDTO.getUserId());
        stammkundeDto.setNewUserName(userDTO.getUserName());
        ShopDTO shop = oopService.shop(customer.getShopId());
        stammkundeDto.setNewShopId(customer.getShopId());
        if (Objects.nonNull(shop)) {
            stammkundeDto.setNewShopName(shop.getShortName());
        }
        stammkundeDto.setGroupId(customer.getGroupId());

        StammkundePool stammkundePool = rejectAndNew(stammkundeDto);
        stammkundePool.setSources(StammkundeSourcesEnum.REDISTRIBUTION);
        stammkundePool.setPoolStatus(StammkundeStatusEnum.TWICE);
        stammkundePool.setAktiv(Boolean.FALSE);
        stammkundePool.setCreateTime(new Date());
        stammkundePool.setGroupId(customer.getGroupId());
        stammkundePoolService.save(stammkundePool);
        eventPublisher.publishEvent(new CustomerDefeatedEvent(task.getClueId(), task.getCustomerId(), task.getType()));
        return stammkundePool;
    }


    /**
     * 更换跟进人
     *
     * @param task
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public PostUserDTO changeInsFollowUser(FollowTask task) throws BusinessException {
        Customer customer = customerService.getById(task.getCustomerId());
        if (Objects.isNull(customer)) {
            log.info("档案信息异常，custId: [{}]", task.getCustomerId());
        }
        BV.notNull(customer, () -> "档案信息不存在");

        List<PostUserDTO> userList = userService.getUserByRole(task.getFollowShop(), RoleCode.XBGJ);
        BV.isNotEmpty(userList, () -> String.format("更换跟进人员失败：更换跟进人员失败：门店[%s]没有续保跟进员", customer.getShopId()));
        Collections.shuffle(userList);
        return userList.stream().filter(u -> !u.getUserId().equals(task.getFollowUser())).findAny().orElse(null);
    }

    @Transactional(rollbackFor = Exception.class)
    public void rollBack(String vin, Long groupId) {
        Customer customer = customerService.queryByFrameNo(vin, groupId);
        BV.notNull(customer, () -> "档案不存在，无需回滚");
        boolean isTemporaryPlateNo = customer.getPlateNo().contains("临") || customer.getPlateNo().contains("商");
        BV.isTrue(isTemporaryPlateNo, () -> "非临时牌照车辆，无法回滚");
        Long customerId = customer.getId();
        List<CustomerCluePool> list = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                .eq(CustomerCluePool::getRefererId, customerId)
                .eq(CustomerCluePool::getGroupId, groupId)
        );
        if (!CollectionUtils.isEmpty(list)) {
            List<Long> clueIds = new ArrayList<>();
            for (CustomerCluePool cluePool : list) {
                boolean isWaiting = ClueStatusEnum.WAITING.equals(cluePool.getClueStatus());
                BV.isTrue(isWaiting, () -> "存在已开始的跟进任务，无法回滚档案");
                clueIds.add(cluePool.getId());
            }
            followTaskService.remove(Wrappers.<FollowTask>lambdaQuery()
                    .eq(FollowTask::getCustomerId, customerId)
                    .in(FollowTask::getClueId, clueIds)
            );
            followRecordService.removeRecordByCustId(customerId, null);
            followNoticeRecordService.removeByClueIds(clueIds);
            customerCluePoolService.removeByIds(clueIds);
        }
        List<StammkundePool> stammkundePoolList = stammkundePoolService.list(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getCustomerId, customerId)
                .eq(StammkundePool::getGroupId, groupId)
        );
        if (!CollectionUtils.isEmpty(stammkundePoolList)) {
            stammkundePoolService.removeByCustomerId(customerId, groupId);
        }
        customerService.forbidden(customerId);
    }

    /**
     * 修改档案
     *
     * @param customer
     * @return
     */
    private Long edit(Customer customer) {
        boolean updated = customerService.updateById(customer);
        if (updated) {
            customerChangeHandler(customer);
        }
        return customer.getId();
    }


    private String createCode(String key, int expires, Long dealerId) {
        final WxBCodeParam param = new WxBCodeParam();
        final Map<String, String> map = new HashMap<>(1);
        map.put("customerKey", key);
        param.setParam(map);
        param.setPage("pgCas/CarArchives/index");
        param.setExpires(expires);
        param.setDealerId(dealerId);
        final byte[] bytes = passportService.getWxBCode(param);
        if (Objects.isNull(bytes)) {
            return null;
        }
        return QRCodeKit.byte2Base64String(bytes);
    }


    /**
     * 消息发送
     *
     * @param targetUserId
     * @param done
     */
    private void sendMsg(Long targetUserId, boolean done, String key) {
        try {
            String text = "档案变更通知";
            final String msgBizType = "customerChangeStatus";
            Map<String, Object> ext = new HashMap<>(3);
            ext.put("type", msgBizType);
            ext.put("done", done);
            ext.put("key", key);
            final MsgParamCondition msgPara = MsgParamCondition.getCustomMsg(text, "", ext,
                    0, targetUserId, false).setBusinessType(BusinessType.INTERNAL_NOTIFICATION).build();
            final Message<Boolean> msg = imSendMessage.synSendMsg(msgPara);
            log.info("给[{}]推送im消息结果:[{}]", targetUserId, msg.getResult());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void changeTaskFollower(Long motoId, Long curId, Long groupId) {
        CustomerCluePool cluePool = customerCluePoolService.queryByRefererId(motoId, groupId, FollowTypeEnum.FM);
        if (Objects.nonNull(cluePool)) {
            boolean equals = ClueStatusEnum.ONGOING.equals(cluePool.getClueStatus()) || ClueStatusEnum.WAITING.equals(cluePool.getClueStatus());
            if (equals) {
                cluePool.setRefererId(curId);
                customerCluePoolService.updateById(cluePool);
            }
        }
        cluePool = customerCluePoolService.queryByRefererId(motoId, groupId, FollowTypeEnum.RM);
        if (Objects.nonNull(cluePool)) {
            boolean equals = ClueStatusEnum.ONGOING.equals(cluePool.getClueStatus()) || ClueStatusEnum.WAITING.equals(cluePool.getClueStatus());
            if (equals) {
                cluePool.setRefererId(curId);
                customerCluePoolService.updateById(cluePool);
            }
        }
        cluePool = customerCluePoolService.queryByRefererId(motoId, groupId, FollowTypeEnum.IR);
        if (Objects.nonNull(cluePool)) {
            boolean equals = ClueStatusEnum.ONGOING.equals(cluePool.getClueStatus()) || ClueStatusEnum.WAITING.equals(cluePool.getClueStatus());
            if (equals) {
                cluePool.setRefererId(curId);
                customerCluePoolService.updateById(cluePool);
            }
        }

        List<FollowTask> list = followTaskService.getListByCustomerId(motoId, null);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        list.forEach(r -> r.setCustomerId(curId));
        followTaskService.updateBatchById(list);
    }

    private void create(Customer customer, Long adviserId, Long shopId) {
        StammkundePool stammkundePool = createSimpleInfo(customer, adviserId);
        stammkundePool.setSources(StammkundeSourcesEnum.NATURAL);
        stammkundePool.setShopId(shopId);
        stammkundePool.setPoolStatus(StammkundeStatusEnum.KUNDE);
        stammkundePool.setAktiv(Boolean.TRUE);
        stammkundePool.setActivationTime(new Date());
        stammkundePool.setCreateTime(new Date());
        stammkundePoolService.save(stammkundePool);
    }

    private AffiliationRecord createEntity(Customer customer, Long adviserId, Long shopId) {
        AffiliationRecord record = new AffiliationRecord();
        record.setCustomerId(customer.getId());
        record.setType(CustomerChangeTypeEnum.DEFEAT);
        record.setDefeatTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
        record.setOriginUserId(customer.getAdviserId());
        record.setOriginShopId(customer.getShopId());
        record.setCurUserId(adviserId);
        record.setCurShopId(shopId);
        record.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
        return record;
    }
}
