package cn.fw.valhalla.service.bus.follow;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.dto.OriginalDataDTO;
import cn.fw.valhalla.sdk.enums.DataTypeEnum;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.OriginalDataService;
import cn.fw.valhalla.service.event.TaskCancelEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.util.Date;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : OriginalDataBizService
 * @description : 跟进原始数据处理服务
 * @date: 2020-08-14 17:47
 */
@Slf4j
@Service
public class OriginalDataBizService {
    private final OriginalDataService originalDataService;
    private final CustomerService customerService;
    private final ApplicationEventPublisher eventPublisher;

    @Autowired
    public OriginalDataBizService(final OriginalDataService originalDataService,
                                  final CustomerService customerService,
                                  final ApplicationEventPublisher eventPublisher) {
        this.originalDataService = originalDataService;
        this.customerService = customerService;
        this.eventPublisher = eventPublisher;
    }

    /**
     * 保存原始数据
     *
     * @param dataDTO
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveOrigin(OriginalDataDTO dataDTO) {
        OriginalData originalData = originalDataService.getOne(Wrappers.<OriginalData>lambdaQuery()
                .eq(OriginalData::getType, dataDTO.getType())
                .eq(OriginalData::getDetailId, dataDTO.getDetailId())
        );
        if (Objects.nonNull(originalData)) {
            return;
        }
        originalData = new OriginalData();
        Customer customer = customerService.queryById(dataDTO.getCustomerId());
        if (Objects.isNull(customer)) {
            return;
        }
        BeanUtils.copyProperties(dataDTO, originalData);
        if (DataTypeEnum.BI.equals(dataDTO.getType())) {
            Timestamp insExpireTime = DateUtil.getExpiredYear(dataDTO.getGenerateTime(), 1);
            customer.setInsuranceExpires(insExpireTime);
            customerService.updateById(customer);
            originalData.setGenerateTime(insExpireTime);
        }
        originalData.setSolved(Boolean.FALSE);
        originalData.setCreateTime(new Date());
        originalData.setUpdateTime(new Date());
        originalDataService.save(originalData);
    }

    /**
     * 删除无效原始数据
     *
     * @param type
     * @param detailId
     */
    @Transactional(rollbackFor = Exception.class)
    public void remove(DataTypeEnum type, String detailId) {
        OriginalData data = originalDataService.getOne(Wrappers.<OriginalData>lambdaQuery()
                .eq(OriginalData::getDetailId, detailId)
                .eq(OriginalData::getType, type)
        );
        if (Objects.nonNull(data)) {
            originalDataService.removeById(data.getId());
            if (Boolean.TRUE.equals(data.getSolved())) {
                TaskCancelEvent event = new TaskCancelEvent(type, data.getCustomerId(), data.getGenerateTime(), data.getGroupId());
                eventPublisher.publishEvent(event);
            }
        }
    }

    /**
     * 修改原始数据标识
     *
     * @param id
     */
    @Transactional(rollbackFor = Exception.class)
    public void solve(Long id) {
        originalDataService.update(Wrappers.<OriginalData>lambdaUpdate()
                .set(OriginalData::getSolved, Boolean.TRUE)
                .eq(OriginalData::getId, id)
        );
    }
}
