package cn.fw.valhalla.service.bus.follow.strategy.impl;

import cn.fw.common.exception.BusinessException;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.MobileUtil;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.follow.FollowRecord;
import cn.fw.valhalla.domain.db.follow.FollowRecordLog;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.dto.FollowAttachmentDTO;
import cn.fw.valhalla.domain.enums.AttTypeEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.vo.follow.FollowDetailVO;
import cn.fw.valhalla.domain.vo.follow.FollowRecordVO;
import cn.fw.valhalla.domain.vo.follow.FollowTodoListVO;
import cn.fw.valhalla.domain.vo.follow.PublicPoolDetailVO;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy;
import cn.fw.valhalla.service.data.PublicCluePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : ACFollowStrategy
 * @description : 事故车策略
 * @date: 2020-08-17 10:48
 */
@Slf4j
@Component
public class OTFollowStrategy extends AbstractFollowStrategy {

    private final PublicCluePoolService publicCluePoolService;

    @Autowired
    public OTFollowStrategy(PublicCluePoolService publicCluePoolService) {
        this.publicCluePoolService = publicCluePoolService;
    }

    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.OT;
    }

    @Override
    public List<FollowTodoListVO> getList(Integer startIndex, Integer pageSize, Long userId) {
        return new ArrayList<>();
    }

    @Override
    public FollowDetailVO getDetail(Long id) {
        PublicCluePool publicCluePool = publicCluePoolService.getById(id);
        BV.notNull(publicCluePool, "记录不存在");
        List<FollowRecord> recordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, id)
                .eq(FollowRecord::getType, FollowTypeEnum.OT)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .orderByAsc(FollowRecord::getId)
        );
        BV.isNotEmpty(recordList, "记录不存在");
        FollowRecord followRecord = recordList.get(0);
        FollowDetailVO vo = assemble(publicCluePool.getCustomerId());
        int count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                .eq(FollowRecordLog::getRecordId, followRecord.getId())
                .eq(FollowRecordLog::getAttType, AttTypeEnum.SMART_PHONE)
        );
        vo.setHadCall(count > 0);
        vo.setId(followRecord.getId());
        vo.setTaskId(followRecord.getTaskId());
        return vo;
    }

    @Override
    public List<FollowRecordVO> getRecordList(Long id) {
        List<FollowRecord> followRecordList = new ArrayList<>();
        PublicCluePool publicCluePool = publicCluePoolService.getById(id);
        if (Objects.isNull(publicCluePool)) {
            return new ArrayList<>();
        }
        List<FollowRecord> recordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getCustomerId, publicCluePool.getCustomerId())
                .eq(FollowRecord::getTaskId, publicCluePool.getId())
        );

        if (!CollectionUtils.isEmpty(recordList)) {
            followRecordList.addAll(recordList);
        }
        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        List<Long> recordIds = followRecordList.stream().map(FollowRecord::getId).collect(Collectors.toList());
        Map<Long, String> map = followRecordList.stream().collect(Collectors.toMap(FollowRecord::getId, FollowRecord::getUserName));
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(recordIds);
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        List<FollowRecordVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecordVO vo = new FollowRecordVO();
            vo.setAttachments(attachment.getAttachments());
            vo.setId(attachment.getId());
            vo.setRecordId(attachment.getRecordId());
            vo.setTaskId(publicCluePool.getId());
            vo.setAttType(attachment.getAttType());
            vo.setFeedbackType(attachment.getFeedbackType());
            vo.setFollowType(getFollowType());
            vo.setUploadTime(attachment.getUploadTime());
            vo.setDescribes(attachment.getDescribes());
            vo.setUserName(map.get(attachment.getRecordId()));
            list.add(vo);
        }
        return list;
    }

    @Override
    public void completeRecordAndEnd(FollowRecord record) {
        final Date followTime = DateUtil.localDateTime2Date(LocalDateTime.now());
        BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), getItemCode(record.getType()), String.valueOf(record.getId()), followTime, record.getShopId());
        if (Boolean.FALSE.equals(record.getOutTime()) && Objects.isNull(record.getFollowTime())) {
            record.setFollowTime(followTime);
            followRecordService.updateById(record);
            todoRpcService.complete(dto);
            return;
        }
        if (record.getLimitTime().after(followTime)) {
            todoRpcService.cancel(dto);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadAtt(FollowAttachmentDTO dto, Long userId) {
        FollowRecord record = followRecordService.getById(dto.getRecordId());
        if (Objects.isNull(record)) {
            return;
        }
        BV.isTrue(record.getUserId().equals(userId), () -> "无法跟进非本人的任务");
        FollowRecordLog recordLog = new FollowRecordLog();
        recordLog.setId(dto.getId());
        recordLog.setTaskId(record.getTaskId());
        recordLog.setAttachments(dto.getAttachments());
        recordLog.setAttType(dto.getAttTypeEnum());
        recordLog.setFeedbackType(dto.getFeedbackTypeEnum());
        recordLog.setDescribes(dto.getDescribes());
        recordLog.setRecordId(dto.getRecordId());
        if (Objects.isNull(dto.getId())) {
            recordLog.setUploadTime(new Date());
        }
        followRecordLogService.saveOrUpdate(recordLog);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cancelFollowTask(Long customerId, Date time, Long groupId) {
    }

    @Override
    public void settingChanged(List<SettingVO> setting, Long groupId) {
    }

    @Override
    public boolean origin2task(OriginalData originalData) {
        return true;
    }

    @Override
    public void updateTask(Long customerId) {
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void overdueProcessing(FollowRecord record) {
        followRecordService.overdue(record.getId());
    }

    @Override
    public FollowDetailVO followPoolDetail(FollowTask task) {
        return null;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void closeTask(FollowTask task) throws BusinessException {
    }

    /**
     * 完成生成的跟进待办
     *
     * @param taskId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    protected void completeTodo(Long taskId) {
    }


    @Override
    public FollowDetailVO assemble(Long customerId) {
        CustomerDetailDto customerDetailDto = customerBizService.queryById(customerId);
        PublicPoolDetailVO vo = new PublicPoolDetailVO();
        vo.setCustomerId(customerId);
        vo.setName(customerDetailDto.getName());
        vo.setMobile(customerDetailDto.getMobile());
        vo.setRegion(MobileUtil.attribution(customerDetailDto.getMobile()));
        vo.setRealMobile(customerDetailDto.getMobile());
        vo.setPlateNo(customerDetailDto.getPlateNo());
        vo.setVin(customerDetailDto.getFrameNo());
        vo.setAdviserId(customerDetailDto.getAdviserId());
        vo.setAdviserName(customerDetailDto.getAdviserName());
        vo.setCarModel(customerDetailDto.getBrandName() + " " + customerDetailDto.getSeriesName());
        vo.setCusLevel(customerDetailDto.getCusLevel());
        vo.setArrivalCount(customerDetailDto.getArrivalCount());
        return vo;
    }
}
