package cn.fw.valhalla.service.bus.setting.strategy;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.valhalla.domain.db.setting.FollowSetting;
import cn.fw.valhalla.domain.db.setting.FollowSettingDetail;
import cn.fw.valhalla.domain.dto.SettingDTO;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.SettingTypeEnum;
import cn.fw.valhalla.domain.enums.SettingUnitEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.service.data.FollowSettingDetailService;
import cn.fw.valhalla.service.data.FollowSettingService;
import cn.fw.valhalla.service.event.SettingChangeEvent;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.redis.core.BoundHashOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : AbstractSettingStrategy
 * @description : 设置策略抽象类
 * @date: 2020-08-14 10:31
 */
@Slf4j
public abstract class AbstractSettingStrategy implements SettingStrategy {
    @Autowired
    protected FollowSettingService settingService;
    @Autowired
    protected FollowSettingDetailService settingDetailService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private DistributedLocker distributedLocker;
    @Autowired
    private ApplicationEventPublisher eventPublisher;
    /**
     * redis key前缀
     */
    @Value("${spring.cache.custom.global-prefix}:setting")
    @Getter
    private String keyPrefix;

    /**
     * 查询列表
     *
     * @param type
     * @param groupId
     * @return
     */
    protected List<SettingVO> getListByCat(final FollowTypeEnum type, final Long groupId) {
        final String key = generateKey(groupId, type);
        List<SettingVO> voList = getFromCache(key);
        if (!CollectionUtils.isEmpty(voList)) {
            return voList;
        }
        FollowSetting setting = settingService.getOne(Wrappers.<FollowSetting>lambdaQuery()
                .eq(FollowSetting::getCategory, type)
                .eq(FollowSetting::getGroupId, groupId)
        );
        if (Objects.isNull(setting)) {
            return new ArrayList<>();
        }
        List<FollowSettingDetail> list = settingDetailService.list(Wrappers.<FollowSettingDetail>lambdaQuery()
                .eq(FollowSettingDetail::getSettingId, setting.getId())
                .eq(FollowSettingDetail::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            new ArrayList<>();
        }
        voList = list.stream().sorted(Comparator.comparingInt(a -> a.getType().getSeq())).distinct().map(SettingVO::with).collect(Collectors.toList());
        setToCache(key, voList);
        return voList;
    }

    /**
     * 查询某个设置类型
     *
     * @param catType
     * @param groupId
     * @param settingType
     * @return
     */
    protected SettingVO getBySettingType(final FollowTypeEnum catType, final Long groupId, final SettingTypeEnum settingType) {
        final String key = generateKey(groupId, catType);
        SettingVO vo = getFromCache(key, settingType);
        if (Objects.nonNull(vo)) {
            return vo;
        }
        List<SettingVO> list = getListByCat(catType, groupId);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        List<SettingVO> settingVOList = list.stream().filter(r -> settingType.getValue().equals(r.getType())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(settingVOList)) {
            return null;
        }
        return settingVOList.get(0);
    }

    /**
     * 保存设置项
     *
     * @param type
     * @param groupId
     * @param list
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    protected boolean saveOrUpdate(final FollowTypeEnum type, final Long groupId, final List<SettingDTO> list) {
        String lockKey = getLockKey(groupId, type);
        Pair<Boolean, RLock> pair = distributedLocker.tryLock(lockKey, TimeUnit.SECONDS, 0, 15);
        BV.isTrue(Boolean.TRUE.equals(pair.getKey()), "请勿重复提交");
        try {
            final String key = generateKey(groupId, type);
            FollowSetting setting = settingService.getOne(Wrappers.<FollowSetting>lambdaQuery()
                    .eq(FollowSetting::getCategory, type)
                    .eq(FollowSetting::getGroupId, groupId)
            );
            if (Objects.isNull(setting)) {
                setting = new FollowSetting();
                setting.setCategory(type);
                setting.setGroupId(groupId);
                setting.setCreateTime(new Date());
                setting.setUpdateTime(new Date());
                settingService.save(setting);
            }
            List<FollowSettingDetail> detailList = new ArrayList<>();
            for (SettingDTO settingDTO : list) {
                FollowSettingDetail detail = new FollowSettingDetail();
                detail.setSettingId(setting.getId());
                detail.setId(settingDTO.getId());
                detail.setDetailValue(settingDTO.getDetailValue());
                if (SettingTypeEnum.REVISE_RATIO.getValue().equals(settingDTO.getType()) || SettingTypeEnum.RETENTION_RATIO.getValue().equals(settingDTO.getType())) {
                    detail.setDetailValue(settingDTO.getDetailValue() * 100);
                }
                detail.setType(SettingTypeEnum.ofValue(settingDTO.getType()));
                detail.setUnit(SettingUnitEnum.ofValue(settingDTO.getUnit()));
                detail.setYn(Boolean.TRUE);
                detail.setGroupId(groupId);
                detail.setUpdateTime(new Date());
                if (settingDTO.getId() == null) {
                    detail.setCreateTime(new Date());
                }
                detailList.add(detail);
            }
            boolean saved = settingDetailService.saveOrUpdateBatch(detailList);
            SettingChangeEvent event = new SettingChangeEvent(type, groupId);
            clearCache(key);
            eventPublisher.publishEvent(event);
            return saved;
        } finally {
            if (pair.getRight().isLocked()) {
                distributedLocker.unlock(lockKey);
            }
        }
    }


    /**
     * 获取该跟进类型的所有设置信息
     *
     * @param key
     * @return
     */
    private List<SettingVO> getFromCache(final String key) {
        List<SettingVO> list = new ArrayList<>();
        try {
            Boolean aBoolean = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(aBoolean)) {
                return new ArrayList<>();
            }
            BoundHashOperations<String, Integer, String> ops = redisTemplate.boundHashOps(key);
            for (SettingTypeEnum typeEnum : SettingTypeEnum.values()) {
                String settingJson = ops.get(String.valueOf(typeEnum.getValue()));
                if (StringUtils.isNotBlank(settingJson)) {
                    SettingVO settingVO = JSONObject.parseObject(settingJson, SettingVO.class);
                    list.add(settingVO);
                }
            }
        } catch (Exception e) {
            log.error("从缓存获设置信息失败 key[{}]", key, e);
        }
        return list;
    }

    /**
     * 获取该跟进类型下特定类型的设置信息
     *
     * @param key
     * @return
     */
    private SettingVO getFromCache(final String key, final SettingTypeEnum settingType) {
        try {
            Boolean aBoolean = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(aBoolean)) {
                return null;
            }
            BoundHashOperations<String, String, String> ops = redisTemplate.boundHashOps(key);
            String settingJson = ops.get(String.valueOf(settingType.getValue()));
            if (StringUtils.isNotBlank(settingJson)) {
                return JSONObject.parseObject(settingJson, SettingVO.class);
            }
        } catch (Exception e) {
            log.error("从缓存获特定类型的设置信息失败： key[{}]  type[{}]", key, settingType.getValue(), e);
        }
        return null;
    }

    /**
     * 缓存设置信息
     *
     * @param key
     * @param list
     */
    private void setToCache(final String key, final List<SettingVO> list) {
        try {
            BoundHashOperations<String, String, String> hashOps = redisTemplate.boundHashOps(key);
            HashMap<String, String> map = new HashMap<>(list.size());
            for (SettingVO settingVO : list) {
                String jsonString = JSONObject.toJSONString(settingVO);
                map.put(String.valueOf(settingVO.getType()), jsonString);
            }
            hashOps.putAll(map);
            redisTemplate.expire(key, 12, TimeUnit.HOURS);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    /**
     * 清除缓存信息
     *
     * @param key
     */
    private void clearCache(final String key) {
        redisTemplate.delete(key);
    }


    /**
     * 生成 Redis Key
     */
    private String generateKey(final Long groupId, final FollowTypeEnum type) {
        Assert.notNull(groupId, "groupId cannot be null");
        Assert.notNull(type, "type cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), groupId, type.name());
    }

    /**
     * 防重锁
     *
     * @param groupId
     * @param type
     * @return
     */
    private String getLockKey(final Long groupId, final FollowTypeEnum type) {
        Assert.notNull(groupId, "groupId cannot be null");
        Assert.notNull(type, "type cannot be null");

        return String.format("%s:lock:%s:%s", getKeyPrefix(), groupId, type.name());
    }
}
